@@ -1,4 +1,4 @@
-# Copyright 2022 Observational Health Data Sciences and Informatics
+# Copyright 2023 Observational Health Data Sciences and Informatics
 #
 # This file is part of DatabaseConnector
 #
@@ -17,24 +17,17 @@
 #' Low level function for retrieving data to a local Andromeda object
 #'
 #' @description
-#' This is the equivalent of the \code{\link{querySqlToAndromeda}} function, except no error report is
+#' This is the equivalent of the [querySqlToAndromeda()] function, except no error report is
 #' written when an error occurs.
 #'
-#' @param connection           The connection to the database server.
+#' @template Connection
 #' @param query                The SQL statement to retrieve the data
 #' @param datesAsString        Should dates be imported as character vectors, our should they be
 #'                             converted to R's date format?
-#' @param andromeda            An open Andromeda object, for example as created using
-#'                             \code{\link[Andromeda]{andromeda}}.
-#' @param andromedaTableName   The name of the table in the local Andromeda object where the results
-#'                             of the query will be stored.
-#' @param integerAsNumeric     Logical: should 32-bit integers be converted to numeric (double) values?
-#'                             If FALSE 32-bit integers will be represented using R's native
-#'                             \code{Integer} class.
-#' @param integer64AsNumeric   Logical: should 64-bit integers be converted to numeric (double) values?
-#'                             If FALSE 64-bit integers will be represented using
-#'                             \code{bit64::integer64}.
-#'
+#' @template Andromeda
+#' @template SnakeCaseToCamelCase
+#' @template IntegerAsNumeric
+#' 
 #' @details
 #' Retrieves data from the database server and stores it in a local Andromeda object This allows
 #' very large data sets to be retrieved without running out of memory. Null values in the database are
@@ -51,6 +44,8 @@ lowLevelQuerySqlToAndromeda <- function(connection,
                                         andromeda,
                                         andromedaTableName,
                                         datesAsString = FALSE,
+                                        appendToTable = FALSE,
+                                        snakeCaseToCamelCase = FALSE,
                                         integerAsNumeric = getOption("databaseConnectorIntegerAsNumeric",
                                                                      default = TRUE
                                         ),
@@ -66,6 +61,8 @@ lowLevelQuerySqlToAndromeda.default <- function(connection,
                                                 andromeda,
                                                 andromedaTableName,
                                                 datesAsString = FALSE,
+                                                appendToTable = FALSE,
+                                                snakeCaseToCamelCase = FALSE,
                                                 integerAsNumeric = getOption("databaseConnectorIntegerAsNumeric",
                                                                              default = TRUE
                                                 ),
@@ -75,6 +72,8 @@ lowLevelQuerySqlToAndromeda.default <- function(connection,
   if (rJava::is.jnull(connection@jConnection)) {
     stop("Connection is closed")
   }
+  logTrace(paste("Querying SQL:", truncateSql(query)))
+  startTime <- Sys.time()
   
   batchedQuery <- rJava::.jnew(
     "org.ohdsi.databaseConnector.BatchedQuery",
@@ -101,14 +100,19 @@ lowLevelQuerySqlToAndromeda.default <- function(connection,
                                  integer64AsNumeric = integer64AsNumeric,
                                  integerAsNumeric = integerAsNumeric
     )
-    
-    if (first) {
+    if (snakeCaseToCamelCase) {
+      colnames(batch) <- SqlRender::snakeCaseToCamelCase(colnames(batch))
+    }
+    if (first && !appendToTable) {
       andromeda[[andromedaTableName]] <- batch
     } else {
       Andromeda::appendToTable(andromeda[[andromedaTableName]], batch)
     }
     first <- FALSE
   }
+  delta <- Sys.time() - startTime
+  logTrace(paste("Querying SQL took", delta, attr(delta, "units")))
+  
   invisible(andromeda)
 }
 
@@ -118,19 +122,51 @@ lowLevelQuerySqlToAndromeda.DatabaseConnectorDbiConnection <- function(connectio
                                                                        andromeda,
                                                                        andromedaTableName,
                                                                        datesAsString = FALSE,
+                                                                       appendToTable = FALSE,
+                                                                       snakeCaseToCamelCase = FALSE,
                                                                        integerAsNumeric = getOption("databaseConnectorIntegerAsNumeric",
                                                                                                     default = TRUE
                                                                        ),
                                                                        integer64AsNumeric = getOption("databaseConnectorInteger64AsNumeric",
                                                                                                       default = TRUE
                                                                        )) {
-  results <- lowLevelQuerySql(connection,
-                              query,
-                              integerAsNumeric = integerAsNumeric,
-                              integer64AsNumeric = integer64AsNumeric
-  )
+  logTrace(paste("Querying SQL:", truncateSql(query)))
+  startTime <- Sys.time()
+  
+  batchSize <- 100000
+  resultSet <- DBI::dbSendQuery(connection@dbiConnection, query)
+  on.exit(DBI::dbClearResult(resultSet))
+  first <- TRUE
+  while (first || !DBI::dbHasCompleted(resultSet)) {
+    batch <- DBI::dbFetch(resultSet, batchSize)
+    if (integerAsNumeric) {
+      for (i in seq_len(ncol(batch))) {
+        if (is(batch[[i]], "integer")) {
+          batch[[i]] <- as.numeric(batch[[i]])
+        }
+      }
+    }
+    if (integer64AsNumeric) {
+      for (i in seq_len(ncol(batch))) {
+        if (is(batch[[i]], "integer64")) {
+          batch[[i]] <- convertInteger64ToNumeric(batch[[i]])
+        }
+      }
+    }
+    batch <- convertFields(dbms(connection), batch)
+    if (snakeCaseToCamelCase) {
+      colnames(batch) <- SqlRender::snakeCaseToCamelCase(colnames(batch))
+    }
+    if (first && !appendToTable) {
+      andromeda[[andromedaTableName]] <- batch
+    } else {
+      Andromeda::appendToTable(andromeda[[andromedaTableName]], batch)
+    }
+    first <- FALSE
+  }
+  delta <- Sys.time() - startTime
+  logTrace(paste("Querying SQL took", delta, attr(delta, "units")))
   
-  andromeda[[andromedaTableName]] <- results
   invisible(andromeda)
 }
 
@@ -139,22 +175,12 @@ lowLevelQuerySqlToAndromeda.DatabaseConnectorDbiConnection <- function(connectio
 #' @description
 #' This function sends SQL to the server, and returns the results in a local Andromeda object
 #'
-#' @param connection             The connection to the database server.
+#' @template Connection
 #' @param sql                    The SQL to be sent.
-#' @param andromeda              An open connection to a Andromeda object, for example as created
-#'                               using \code{\link[Andromeda]{andromeda}}.
-#' @param andromedaTableName     The name of the table in the local Andromeda object where the
-#'                               results of the query will be stored.
-#' @param errorReportFile        The file where an error report will be written if an error occurs.
-#'                               Defaults to 'errorReportSql.txt' in the current working directory.
-#' @param snakeCaseToCamelCase   If true, field names are assumed to use snake_case, and are converted
-#'                               to camelCase.
-#' @param integerAsNumeric       Logical: should 32-bit integers be converted to numeric (double)
-#'                               values? If FALSE 32-bit integers will be represented using R's native
-#'                               \code{Integer} class.
-#' @param integer64AsNumeric     Logical: should 64-bit integers be converted to numeric (double)
-#'                               values? If FALSE 64-bit integers will be represented using
-#'                               \code{bit64::integer64}.
+#' @template ErrorReportFile
+#' @template SnakeCaseToCamelCase
+#' @template Andromeda
+#' @template IntegerAsNumeric
 #'
 #' @details
 #' Retrieves data from the database server and stores it in a local Andromeda object. This allows
@@ -195,6 +221,7 @@ querySqlToAndromeda <- function(connection,
                                 andromedaTableName,
                                 errorReportFile = file.path(getwd(), "errorReportSql.txt"),
                                 snakeCaseToCamelCase = FALSE,
+                                appendToTable = FALSE,
                                 integerAsNumeric = getOption("databaseConnectorIntegerAsNumeric",
                                                              default = TRUE
                                 ),
@@ -230,24 +257,26 @@ querySqlToAndromeda <- function(connection,
         query = sqlStatements[1],
         andromeda = andromeda,
         andromedaTableName = andromedaTableName,
+        appendToTable = appendToTable, 
+        snakeCaseToCamelCase = snakeCaseToCamelCase,
         integerAsNumeric = integerAsNumeric,
         integer64AsNumeric = integer64AsNumeric
       )
-      if (inherits(andromeda, "SQLiteConnection")) {
-        columnNames <- colnames(andromeda[[andromedaTableName]])
-        if (snakeCaseToCamelCase) {
-          newColumnNames <- SqlRender::snakeCaseToCamelCase(columnNames)
-        } else {
-          newColumnNames <- toupper(columnNames)
-        }
-        names(andromeda[[andromedaTableName]]) <- newColumnNames
-      } else {
-        if (snakeCaseToCamelCase) {
-          andromeda[[andromedaTableName]] <- dplyr::rename_with(andromeda[[andromedaTableName]], SqlRender::snakeCaseToCamelCase)
-        } else {
-          andromeda[[andromedaTableName]] <- dplyr::rename_with(andromeda[[andromedaTableName]], toupper)
-        }
-      }
+      # if (inherits(andromeda, "SQLiteConnection")) {
+      #   columnNames <- colnames(andromeda[[andromedaTableName]])
+      #   if (snakeCaseToCamelCase) {
+      #     newColumnNames <- SqlRender::snakeCaseToCamelCase(columnNames)
+      #   } else {
+      #     newColumnNames <- toupper(columnNames)
+      #   }
+      #   names(andromeda[[andromedaTableName]]) <- newColumnNames
+      # } else {
+      #   if (snakeCaseToCamelCase) {
+      #     andromeda[[andromedaTableName]] <- dplyr::rename_with(andromeda[[andromedaTableName]], SqlRender::snakeCaseToCamelCase)
+      #   } else {
+      #     andromeda[[andromedaTableName]] <- dplyr::rename_with(andromeda[[andromedaTableName]], toupper)
+      #   }
+      # }
       invisible(andromeda)
     },
     error = function(err) {
@@ -262,28 +291,18 @@ querySqlToAndromeda <- function(connection,
 #' This function renders, and translates SQL, sends it to the server, and returns the results as an
 #' ffdf object
 #'
-#' @param connection             The connection to the database server.
+#' @template Connection
 #' @param sql                    The SQL to be send.
-#' @param andromeda              An open Andromeda object, for example as created
-#'                               using \code{\link[Andromeda]{andromeda}}.
-#' @param andromedaTableName     The name of the table in the local Andromeda object where the
-#'                               results of the query will be stored.
-#' @param errorReportFile        The file where an error report will be written if an error occurs.
-#'                               Defaults to 'errorReportSql.txt' in the current working directory.
-#' @param snakeCaseToCamelCase   If true, field names are assumed to use snake_case, and are converted
-#'                               to camelCase.
+#' @template ErrorReportFile
+#' @template SnakeCaseToCamelCase
+#' @template Andromeda
 #' @template TempEmulationSchema
-#' @param integerAsNumeric       Logical: should 32-bit integers be converted to numeric (double)
-#'                               values? If FALSE 32-bit integers will be represented using R's native
-#'                               \code{Integer} class.
-#' @param integer64AsNumeric     Logical: should 64-bit integers be converted to numeric (double)
-#'                               values? If FALSE 64-bit integers will be represented using
-#'                               \code{bit64::integer64}.
+#' @template IntegerAsNumeric
 #' @param ...                    Parameters that will be used to render the SQL.
 #'
 #' @details
-#' This function calls the \code{render} and \code{translate} functions in the SqlRender package
-#' before calling \code{\link{querySqlToAndromeda}}.
+#' This function calls the `render` and `translate` functions in the `SqlRender` package
+#' before calling [querySqlToAndromeda()].
 #'
 #' @return
 #' Invisibly returns the andromeda. The Andromeda object will have a table added with the query
@@ -319,6 +338,7 @@ renderTranslateQuerySqlToAndromeda <- function(connection,
                                                  "errorReportSql.txt"
                                                ),
                                                snakeCaseToCamelCase = FALSE,
+                                               appendToTable = FALSE,
                                                oracleTempSchema = NULL,
                                                tempEmulationSchema = getOption("sqlRenderTempEmulationSchema"),
                                                integerAsNumeric = getOption("databaseConnectorIntegerAsNumeric",
@@ -328,6 +348,10 @@ renderTranslateQuerySqlToAndromeda <- function(connection,
                                                                               default = TRUE
                                                ),
                                                ...) {
+  if (is(connection, "Pool")) {
+    connection <- pool::poolCheckout(connection)
+    on.exit(pool::poolReturn(connection))
+  }
   if (!is.null(oracleTempSchema) && oracleTempSchema != "") {
     warn("The 'oracleTempSchema' argument is deprecated. Use 'tempEmulationSchema' instead.",
          .frequency = "regularly",
@@ -347,6 +371,7 @@ renderTranslateQuerySqlToAndromeda <- function(connection,
     andromedaTableName = andromedaTableName,
     errorReportFile = errorReportFile,
     snakeCaseToCamelCase = snakeCaseToCamelCase,
+    appendToTable = appendToTable,
     integerAsNumeric = integerAsNumeric,
     integer64AsNumeric = integer64AsNumeric
   ))
